package com.edifixio.bms.controllers;

import java.io.FileInputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;
import javax.faces.validator.ValidatorException;

//import org.richfaces.model.ConvertableKeyModel;

import com.edifixio.bms.beans.UserBean;
import com.edifixio.bms.dao.BillCategoryDAO;
import com.edifixio.bms.dao.BillHistoryDAO;
import com.edifixio.bms.dao.BillLogDAO;
import com.edifixio.bms.dao.ParamMasterDAO;
import com.edifixio.bms.dao.VendorMasterDAO;
import com.edifixio.bms.lognerror.*;
import com.edifixio.bms.mailModule.MailSender;
import com.edifixio.bms.model.BillBean;
import com.edifixio.bms.model.BillCategory;
import com.edifixio.bms.model.BillHistory;
import com.edifixio.bms.model.BillLog;
import com.edifixio.bms.model.ParamMaster;
import com.edifixio.bms.model.VendorMaster;
import com.edifixio.bms.model.WorkflowBean;
import com.edifixio.bms.model.WorkflowHistory;
import com.edifixio.bms.services.CMSService;
import com.edifixio.bms.util.BMSConstants;
import com.edifixio.bms.util.BMSResourceBundle;
import com.edifixio.bms.util.CommonUtils;
import com.edifixio.bms.util.DateConverter;
import com.edifixio.bms.util.IDGenerator;
import com.edifixio.bms.util.SessionUtil;
import com.edifixio.bms.utility.ldap.LDAPManager;
import com.edifixio.bms.webservice.client.WebServiceClient;
@ManagedBean(name="billController")
@SessionScoped
public class BillController {
	
	private static final String FIRST  = "first";
	private static final String SECOND = "second";
	private static final String THIRD  = "third";
	//private static final String COMMA  = ",";
	
	private static final String BUTTON_FINISH         = "finish";
	private static final String BUTTON_CANCEL         = "cancel";
	
	private static final String DATE_PATTERN          = BMSConstants.CALENDER_DATE_PATTERN;
	private static final String CALENDER_DATE_PATTERN          = BMSConstants.CALENDER_DATE_PATTERN;
	public static final String CURRENCY_FORMATE ="billAmountCurrenyFormate";
	public static final String CURRENCY_FORMATE_HEADING ="INR";
	public static final String USER_MANAGER=BMSConstants.ROLE_MGNR;
	public static final String USER_ACCOUNTANT=BMSConstants.ROLE_ACCT;
	
    public static final SimpleDateFormat DATE_FORMAT  = BMSConstants.DATE_FORMAT;
    public static final DecimalFormat DECIMAL_FOMATER = new DecimalFormat("0.00");
   
	private Date billDate;
	private BillLog  currentBillLog;
	private BillBean currentBill;
	private WorkflowHistory currentWorkflowHistory = null;
	//private WebServiceClient webService = new WebServiceClient();
	
	private boolean uploadEnabled  = false;
	private boolean summaryEnabled = false;
	private String currentTab      = FIRST;
	private BillLogDAO logDao      = new BillLogDAO();
	private int logId              = 0;
	private String errorMsg="";
	
	private  List<SelectItem> categories     = new ArrayList<SelectItem>();
	private List<SelectItem> projects       = new ArrayList<SelectItem>();
	private List<SelectItem> financialYears = new ArrayList<SelectItem>();	
	private List<String> roleList                 = new ArrayList<String>();
	private List<VendorMaster> vendorList   = new VendorMasterDAO().findAll();	
	private List<SelectItem> currencyFormate     = null;//
	
	private List<SelectItem> parties=new ArrayList<SelectItem>(0);
	private List<SelectItem> approvers = null;
	private List<SelectItem> accountingHeads;
	private static final List<SelectItem> approvalTypes = new ArrayList<SelectItem>(0);

	private void init(){	    
	   // String userName =UserController.getCurrentInstance().getCurrentUserName(); //sumit
	    categories.add(new SelectItem(BMSConstants.CATEGORY_HEADING, BMSConstants.CATEGORY_HEADING));
	    List<BillCategory> categoryList = new BillCategoryDAO().findAll();
	    if(categoryList != null){
	        for(BillCategory category:categoryList){            
	            categories.add(new SelectItem(category.getCategoryName(), category.getCategoryName()));
	        }    
	    }
	   
	    financialYears.add(new SelectItem(BMSConstants.YEAR_HEADING, BMSConstants.YEAR_HEADING));
	    List<String> years = DateConverter.getFinancialYears();
	    for(String finYear : years){
	        financialYears.add(new SelectItem(finYear, finYear));
	    }
	    approvalTypes.add(new SelectItem(WorkflowBean.STATE_APPLIED, WorkflowBean.STATE_APPLIED));
        approvalTypes.add(new SelectItem(WorkflowBean.STATE_SAVED, WorkflowBean.STATE_SAVED));
        accountingHeads=new CommonUtils().getAccountingHeadList();
        parties.add(new SelectItem(BMSConstants.SELECT,BMSConstants.SELECT));
       // parties.add(new SelectItem(userName,userName)); //sumit
	}
	
	public List<SelectItem> getApprovalTypes() {
        return approvalTypes;
    }
	
    private void populateProjectAndRole() {
        projects.clear();
        roleList.clear();
        projects.add(new SelectItem(BMSConstants.PROJ_ASSIGN_HEADING, BMSConstants.PROJ_ASSIGN_HEADING));
        ParamMasterDAO paramDAO = new ParamMasterDAO();        
        populateItemList(paramDAO.findAll(BMSConstants.PROJ_TYPE));
        populateItemList(paramDAO.findAll(BMSConstants.ASSIGNMENT_TYPE));
        List<ParamMaster> roles = paramDAO.findAll(BMSConstants.APPROVER_TYPE);
        if(roles != null){
            for(ParamMaster param:roles){    
                roleList.add(param.getParamName());
            }
        }
       
    }
    private List<String> populateCurrencyFormateList()
    {
    	  currencyFormate     = new ArrayList<SelectItem>();
    	  List<String> currencyFormateList=new ArrayList<String>();    	
    	  ResourceBundle resource = BMSResourceBundle.getCurrencyFormateBundle();
    	  Enumeration<String>  keys = resource.getKeys();
    	  while (keys.hasMoreElements()) 
    	  {
    		  String key = (String) keys.nextElement();    	   
    		  currencyFormateList.add(resource.getString(key).trim());
    	  }
    	  currencyFormate.add(new SelectItem(CURRENCY_FORMATE_HEADING,CURRENCY_FORMATE_HEADING));  	
    	  if(currencyFormateList!=null)
    	  {
    		  for(int i=0;i<currencyFormateList.size();i++){
    			currencyFormate.add(new SelectItem(currencyFormateList.get(i)));
    		  }
    	}
    	return(currencyFormateList);
    }
    
    private void populateItemList(List<ParamMaster> paramList) {
        if(paramList != null){
            for(ParamMaster param:paramList){    
                projects.add(new SelectItem(param.getParamCode(), param.getParamName()));
            }
        }
    }
    public String clearAndGotoANewBill()
	{
		currentBill=null;
		setSummaryEnabled(false);
		setUploadEnabled(false);
		setCurrentTab(FIRST);
		clearUploadedFiles();
		return "/NewBill?faces-redirect=true";
	}
    
	public BillController(){
		init();
	    populateProjectAndRole();
	    populateCurrencyFormateList();
	    List<SelectItem> approverList=new ArrayList<SelectItem>();
	    approverList.add(new SelectItem(BMSConstants.APPROVER_HEADING,BMSConstants.APPROVER_HEADING));	
	    
	    String user = UserController.getCurrentInstance().getCurrentUser();
	    getApproverListForSpecificUser(user);
	}
	public List<SelectItem> getApproverListForSpecificUser(String user)
	{
		approvers = new ArrayList<SelectItem>();	    
	    approvers.add(new SelectItem(BMSConstants.APPROVER_HEADING,BMSConstants.APPROVER_HEADING));
	    ArrayList<UserBean> managers = LDAPManager.getLDAPManager().getUsersFromRole(USER_MANAGER);
	    //ArrayList<UserBean> accountants = LDAPManager.getLDAPManager().getUsersFromRole(USER_ACCOUNTANT);
	    ArrayList<String>  usrIdAndNameList=new ArrayList<String>();
	    
	    if(managers != null && managers.size()> 0) 
	    {
	    	for(int i=0;i<managers.size();i++){
	    		if(managers.get(i)!=null)
	    	usrIdAndNameList.add(managers.get(i).getUsrId()+","+managers.get(i).getUsrFullName());
	    	}
	    }
	    if(usrIdAndNameList != null && usrIdAndNameList.size()> 0) 
	    {
	    	for(int k=0;k<usrIdAndNameList.size();k++)
	    	{
	    		String usrId=usrIdAndNameList.get(k).split(",")[0];
	    		String usrName=usrIdAndNameList.get(k).split(",")[1];
	    		if(!user.equalsIgnoreCase(usrId))
	    			approvers.add(new SelectItem(usrId,usrName));
	    	}
	    	
	    }
	    return approvers;
	}
	
	public BillBean getCurrentBill(){		
		currentBillLog = new BillLog();		
		if(currentBill == null){
			currentBill = new BillBean();
			try{
				currentBillLog.setBillLogId(IDGenerator.getUniqueBillId());	
			}catch(Exception e){
			}	
			currentBill.setBillID(currentBillLog.getBillLogId());
			currentBill.setSubmissionDate(DateConverter.getStrDate(new Date()));
			try 
			{
				currentBill.setApproverFullName(LDAPManager.getLDAPManager().getSingleUserName(currentBill.getApproverName()));
			} 
			catch (Exception e) 
			{
				BMSErrorLogger.logError("ERRBMS0000011");
			}
			SessionUtil.setCurrnetBill(currentBill);
			
			this.uploadEnabled = false;
			this.summaryEnabled = false;
	
			logDao.saveOrUpdateBillLog(currentBillLog);
			logId = currentBillLog.getLogId();
		}		
		return currentBill;
	}
	
	public boolean isUploadEnabled() {
		return uploadEnabled;
	}

	public void setUploadEnabled(boolean uploadEnabled) {
		this.uploadEnabled = uploadEnabled;
	}

	public boolean isSummaryEnabled() {
		return summaryEnabled;
	}

	public void setSummaryEnabled(boolean summaryEnabled) {
		this.summaryEnabled = summaryEnabled;
	}
	
	public void change(ValueChangeEvent event) {
	    //String userName = webService.getUserName(UserController.getCurrentInstance().getCurrentUser());
	    String userName =UserController.getCurrentInstance().getCurrentUserName();
	    String billType = (String) event.getNewValue(); 
	   // System.out.println("Current user:"+userName+" Current Selection:"+billType+" "+categories.get(1).getLabel());
	    parties.clear();
	    parties.add(new SelectItem(BMSConstants.SELECT,BMSConstants.SELECT));
	    this.currentBill.setVendorPAN(null);
	    if(billType.equals(categories.get(1).getLabel())){
	        currentBill.setPartyName(userName);
	        parties.add(new SelectItem(userName,userName));
	    }else{
	        if(vendorList != null){
	            for(VendorMaster vendor:vendorList){    
	                parties.add(new SelectItem(vendor.getVendorName(),vendor.getVendorName()));
	            }    
	        }  
	    }
	    FacesContext.getCurrentInstance().getRenderResponse();
	}
	public void changeVendor(ValueChangeEvent event) {
	    String vendor = (String) event.getNewValue(); 
	    
	    if(vendorList != null){
	    	for(int i=0;i<vendorList.size();i++){
	    		if(vendorList.get(i).getVendorCode().equalsIgnoreCase(vendor) || vendorList.get(i).getVendorName().equalsIgnoreCase(vendor)){
	    			this.currentBill.setVendorPAN(vendorList.get(i).getVendorPAN());
	    			break;
	    		}
	    	}
	    }
	    FacesContext.getCurrentInstance().getRenderResponse();
	}
	
	public String submit(){
	    return "";
	}

	public String saveData(){
	    String user  = UserController.getCurrentInstance().getCurrentUser();
	    //String owner = webService.getUserName(user);
	    currentBill.setStatus(WorkflowBean.STATE_SAVED);
	    currentBill.setOwnerId(user);
	    try {
			currentBill.setOwner(LDAPManager.getLDAPManager().getSingleUserName(user));
		} catch (Exception e1) {
			e1.printStackTrace();
		}
	    currentBill.setBillDate(getBillDateAsString());
		String billdate=currentBill.getBillDate();
		String financialyear=currentBill.getFinancialYear();
		try 
		{
			currentBill.setApproverFullName(LDAPManager.getLDAPManager().getSingleUserName(currentBill.getApproverName()));
		} 
		catch (Exception e) 
		{
			BMSErrorLogger.logError("ERRBMS0000011");
		}
		try {
			if(DateConverter.checkBillDateWithinFinancialYear(billdate,financialyear,currentBill.getSubmissionDate())){
				String billYear=DateConverter.getFinacialYearPrefix(financialyear);
				CMSService cmsService = new CMSService();
			    cmsService.createBillContent(currentBill,billYear);
			    cmsService.addBillData(currentBill,billYear);
			    
			    setUploadEnabled(true);
			    setCurrentTab(SECOND);
			}
			else
			{
				this.errorMsg="Invalid Bill Date For Your Selected Financial Year.";
				setCurrentTab(FIRST);
			}
		} catch (ParseException e) {
			BMSErrorLogger.logError("ERRBMS0000026");
		}
		
		return "";
	}
	
	public void validateFinancialYear(FacesContext context, UIComponent component, Object input) throws ValidatorException {
	    String element = (String)input;          
	    if(element.equals(BMSConstants.YEAR_HEADING)){
	        throw new ValidatorException(new FacesMessage("Choose proper financial year.", null));
	    }	   
	} 
	
	public void saveFiles(ActionEvent event){
		System.out.println("within saveFiles()");
		clearUploadedFiles();
		System.out.println("files cleared");
		setSummaryEnabled(true);
		setCurrentTab(THIRD);
		System.out.println("Tab set");
	}
	public String getCurrentTab() {
		return currentTab;
	}

	public void setCurrentTab(String currentTab) {
		this.currentTab = currentTab;
	}
	
	public WorkflowHistory getCurrentWorkflowHistory(){
		/*if(currentWorkflowHistory == null){
			this.currentWorkflowHistory = new WorkflowHistory();
			
			WorkflowBean wf = new WorkflowBean();
			//String owner = webService.getUserName(UserController.getCurrentInstance().getCurrentUser());
			wf.setState(currentBill.getStatus());
			wf.setUser(UserController.getCurrentInstance().getCurrentUser());
			try {
				wf.setUserFullName(LDAPManager.getLDAPManager().getSingleUserName(UserController.getCurrentInstance().getCurrentUser()));
			} catch (Exception e) {
				e.printStackTrace();
			}
			wf.setDate(new Date());
			
			this.currentWorkflowHistory.addWorkflowBean(wf);
		}*/
		stateChangeToCurrentWorkflowHistory(currentBill.getStatus());
		return this.currentWorkflowHistory;
	}
	private void stateChangeToCurrentWorkflowHistory(String state){
		if(currentWorkflowHistory == null){
			this.currentWorkflowHistory = new WorkflowHistory();
		}	
		WorkflowBean wf = new WorkflowBean();
		//String owner = webService.getUserName(UserController.getCurrentInstance().getCurrentUser());
		wf.setState(state);
		wf.setUser(UserController.getCurrentInstance().getCurrentUser());
		try {
			wf.setUserFullName(LDAPManager.getLDAPManager().getSingleUserName(UserController.getCurrentInstance().getCurrentUser()));
		} catch (Exception e) {
			e.printStackTrace();
		}
		wf.setDate(new Date());
		
		this.currentWorkflowHistory.addWorkflowBean(wf);
	}
	
	
	public void setCurrentWorkflowHistory(WorkflowHistory currentWorkflowHistory) {
		this.currentWorkflowHistory = currentWorkflowHistory;
	}

	public String finish(){
		
		//save workflow history in cms.
		
		saveBill();
		return BUTTON_FINISH;
	}
	private void saveBill(){
		CMSService cmsService = new CMSService();
		String finacialYearFromBean = currentBill.getFinancialYear();
		String finacialYaer = DateConverter.getFinacialYearPrefix(finacialYearFromBean);
		
		cmsService.addBillWorkflow(currentBill.getBillID(),currentWorkflowHistory,finacialYaer);
		//save bill history in DB.
		BillHistory billHistory = new BillHistory();
		String billId = currentBill.getBillID();
		billHistory.setBillId(billId);
		String user = UserController.getCurrentInstance().getCurrentUser();
		//String owner = webService.getUserName(user);
		billHistory.setActionDoneBy(user);
		billHistory.setBillAmount(Double.parseDouble(BillController.DECIMAL_FOMATER.format(Double.parseDouble(currentBill.getBillAmount()))));//
		billHistory.setCurrentStatus(currentBill.getStatus());
		billHistory.setFinancialYear(currentBill.getFinancialYear());
		billHistory.setLastActionDate(new Date());
		billHistory.setSubmissionDate(new Date());		
		billHistory.setBillDate(getBillDate());
		billHistory.setParty(currentBill.getPartyName());
		billHistory.setOwner(currentBill.getOwner());
		billHistory.setApprover(currentBill.getApproverName());		
		billHistory.setBillCurrencyType(currentBill.getBillCurrencyType());//
		billHistory.setProjectCode(currentBill.getProjectCode());//
		billHistory.setAccountingHead(currentBill.getAccountingHead());
		new BillHistoryDAO().saveOrUpdateBillHistory(billHistory);
		currentBillLog.setLogId(logId);
		currentBillLog.setBillLogId(currentBill.getBillID());
		currentBillLog.setBillCurrentStatus(BillLog.STATUS_SUCCESSFUL);
		currentBillLog.setComment(BillLog.COMMENT_SUBMITTED);
		logDao.saveOrUpdateBillLog(currentBillLog); //updating db		
		
	    closeBillController();
	}
	public String applyBill(){
		currentBill.setStatus(WorkflowBean.STATE_APPLIED);
		stateChangeToCurrentWorkflowHistory(WorkflowBean.STATE_APPLIED);
		MailSender.getMailSender().sendBMSMail(WorkflowBean.STATE_APPLIED, 
				currentBill.getBillID(), currentBill.getOwner(), null, currentBill.getApproverName());
		saveBill();
		return BUTTON_FINISH;
	}
    public String cancel(){	
    	
    	CMSService cmsService = new CMSService();
    	cmsService.deleteBillContent(currentBill);
    	summaryEnabled=false;
    	uploadEnabled=false;
    	currentBillLog.setLogId(logId);
    	currentBillLog.setBillLogId(currentBill.getBillID());
		currentBillLog.setBillCurrentStatus(BillLog.STATUS_UNSUCCESSFUL);
		currentBillLog.setComment(BillLog.COMMENT_CANCELLED);
		logDao.saveOrUpdateBillLog(currentBillLog); //updating db
		closeBillController();
		setCurrentTab(FIRST);
		return BUTTON_CANCEL;
	}
    
    private void closeBillController(){
    	
    	clearUploadedFiles();
    	
    	this.logId = 0;
    	this.currentBill = null;
		this.currentBillLog = null;
		this.currentTab = FIRST;
		
		this.currentWorkflowHistory = null;
    }
    
    private void clearUploadedFiles(){
    	System.out.println("within clearUploadedFiles()");
    	FileUploadController uploader = (FileUploadController)FacesContext.getCurrentInstance().
		getExternalContext().getSessionMap().get("fileUploadController");
    	System.out.println("File Bean received");
    	if(uploader != null)
    		uploader.clearUploadData();
    }

    public List<SelectItem> getCategories() {
        return categories;
    }
    
    public String searchApprover(){
        return "";
    }
    
    public String searchProject(){
        return "";
    }

    public List<SelectItem> getFinancialYears() {
        return financialYears;
    }

    public void setCurrentBill(BillBean currentBill) {
        this.currentBill = currentBill;
    }
    
    public String getDatePattern() {
        return CALENDER_DATE_PATTERN;
    }

    public Date getBillDate() {
    	
        return billDate;
    }
    
    public String getBillDateAsString(){
        Date date = getBillDate();
        return DATE_FORMAT.format(date);
    }
   
    public void setBillDate(Date billDate) {
    	
        this.billDate = billDate;
    }

    public List<SelectItem> getProjects() {
        return projects;
    }

    public  List<SelectItem> getParties() {
        return parties;
    }

    public  void setParties(List<SelectItem> parties) {
        this.parties = parties;
    }

    public List<SelectItem> getApprovers() {
        return approvers;
    }

	public static String getCURRENCY_FORMATE() {
		return CURRENCY_FORMATE;
	}
	public List<SelectItem> getCurrencyFormate() {
		return currencyFormate;
	}
	public String getErrorMsg() {
		return errorMsg;
	}
	public void setErrorMsg(String errorMsg) {
		this.errorMsg = errorMsg;
	}
	public List<SelectItem> getAccountingHeads() {
		return accountingHeads;
	}
}
